//
//  ApplicationController.m
//  GlobalRetainCount
//
//  Created by Patrick Geiller on 12/06/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "ApplicationController.h"
#import "RegExJS.h"

@implementation ApplicationController

- (void)awakeFromNib
{
	// Load our html view
	NSString* path = [[NSBundle mainBundle] pathForResource:@"match" ofType:@"html"];
	NSURL* url = [NSURL fileURLWithPath:path];
	
	// We'll be our own frame load delegate and receive didFinishLoadForFrame
	[webView setFrameLoadDelegate:self];
	[[webView mainFrame] loadRequest:[NSURLRequest requestWithURL:url]];
	[webView setPolicyDelegate:self];
	
	// Update on text change
	[textView setDelegate:self];
	[textPattern setDelegate:self];

	// Test JS match, match all, replace
	[self test:nil];
}


//
//	Javascript is available
//		Register our custom javascript object in the hosted page
//
- (void)webView:(WebView *)view windowScriptObjectAvailable:(WebScriptObject *)windowScriptObject
{
	[windowScriptObject setValue:self forKey:@"ApplicationController"];
	
	[self performSelector:@selector(doMatch) withObject:nil afterDelay:0];
}

//
// Delayed match on launch
//
- (void)doMatch
{
	[self match:nil];
}

//
// URL open
//
- (void)webView:(WebView *)webView decidePolicyForNavigationAction:(NSDictionary *)actionInformation
                                                           request:(NSURLRequest *)request
                                                             frame:(WebFrame *)frame
                                                  decisionListener:(id<WebPolicyDecisionListener>)listener
{
	NSLog(@"GOT HERE");
	[[NSWorkspace sharedWorkspace] openURL:[request URL]];
}


//
// Match input string, send result to webView for display
//
- (IBAction)match:(id)sender
{
	id o = [RegExJSController sharedController];

	NSString* stringToMatch = [textView string];
	id r = [o indicesFromMatching:stringToMatch withPattern:[textPattern stringValue]];
//	NSLog(@"%@->", r);
	
	NSArray* args = [NSArray arrayWithObjects:stringToMatch, r, nil];
	[[webView windowScriptObject] callWebScriptMethod:@"match" withArguments:args];

}

//
// TextView changed
//
- (void)textDidChange:(NSNotification *)aNotification
{
	[self match:nil];
}
//
// TextField changed
//
- (void)controlTextDidChange:(NSNotification *)obj
{
	[self match:nil];
}


//
// Custom matches
//
- (void)customMatch:(NSString*)match
{
	[textPattern setStringValue:match];
	[self match:nil];
}
- (IBAction)matchWords:(id)sender
{
	[self customMatch:@"[a-zA-Z]{3,}"];
}
- (IBAction)matchNumbers:(id)sender
{
	[self customMatch:@"[0-9]+"];
}
- (IBAction)matchSpaces:(id)sender
{
	[self customMatch:@"[\\s]+"];
}
- (IBAction)matchPunctuation:(id)sender
{
	[self customMatch:@"[\\!\\?\\.\\;\\:\\;\\'—\\(\\)…,\\/]"];
}
- (IBAction)matchIPAdresses:(id)sender
{
	[self customMatch:@"[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}"];
}
- (IBAction)matchURLs:(id)sender
{
	[self customMatch:@"(http://)?[a-zA-Z][\\w0-9-]+\\.[a-zA-Z][\\w0-9-]+\\.[a-zA-Z][\\w0-9-]+((/|[\\w_\\.:]+)+)?"];
}
- (IBAction)matchBasicEmails:(id)sender
{
	[self customMatch:@"[\\w]+@[\\w]+\\.[\\w]+"];
}
- (IBAction)matchFrenchWords:(id)sender
{
	[self customMatch:@"[\\w]*(?=[éèçàâô])[\\wéèçàâô]+"];	
}
- (IBAction)matchGermanWords:(id)sender
{
	[self customMatch:@"[\\w]*(?=[öäüß])[\\wöäüß]+"];	
}
- (IBAction)matchKataKana:(id)sender
{
	[self customMatch:@"[\\u30a0-\\u30ff]+"];
}



- (IBAction)test:(id)sender
{
	NSString* source = @"Ceci est un teeeest de replace, case sensitive ET insensitive.";
	
	NSLog(@"%@", [source matchWithPattern:@"E[\\w]+"]);
	NSLog(@"%@", [source matchWithPattern:@"E[\\w]+" isCaseSensitive:NO]);

	NSLog(@"%@", [source matchAllWithPattern:@"E[\\w]+"]);
	NSLog(@"%@", [source matchAllWithPattern:@"E[\\w]+" isCaseSensitive:NO]);

	NSLog(@"replace %@", [source replaceWithString:@"!!!" andPattern:@"E[\\w]+"]);
	NSLog(@"replace %@", [source replaceWithString:@"!!!" andPattern:@"e[\\w]+" isCaseSensitive:NO]);
}



@end
