//
//  RegExJS.m
//  RegEx
//
//  Created by Patrick Geiller on 15/06/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "RegExJS.h"


@implementation RegExJSController

//
// We'll use one shared Javascript engine to do our matching
//
+ (id)sharedController
{
	static id singleton;
	@synchronized(self)
	{
		if (!singleton)
			singleton = [[RegExJSController alloc] init];
		return singleton;
	}
	return singleton;
}

//
// Create Javascript match scripts
//
- (id)init
{
	id o = [super init];

    ctx = JSGlobalContextCreate(NULL);
	
	
	//
	// Match Script
	//	loop over results and record indices
	//
	NSString* indicesScript = @"var p = new RegExp(pattern, flags); \
		var matched = true;			\
		var results = [];			\
		while (matched)				\
		{							\
			var matched = p.test(string);	\
			if (matched)		\
			results.push(new Array(p.lastIndex-RegExp.lastMatch.length, p.lastIndex));	\
			if (matched && RegExp.lastMatch.length == 0)	break	\
		}							\
	results;";

	indicesScriptJS = JSStringCreateWithCFString((CFStringRef)indicesScript);


	NSString* matchScript = @"var p = new RegExp(pattern, flags); string.match(p)";
	matchScriptJS = JSStringCreateWithCFString((CFStringRef)matchScript);

	NSString* replaceScript = @"var p = new RegExp(pattern, flags); string.replace(p, replacement)";
	replaceScriptJS = JSStringCreateWithCFString((CFStringRef)replaceScript);
	
	return	o;
}

//
// Helper method to add a string constant to Javascript
//
- (void)setConstantStringValue:(NSString*)value forName:(NSString*)name
{
    JSStringRef nameJS = JSStringCreateWithCFString((CFStringRef)name);
	JSStringRef valueJS = JSStringCreateWithCFString((CFStringRef)value);

	JSValueRef exception = NULL;
	JSObjectSetProperty(ctx, JSContextGetGlobalObject(ctx), nameJS, JSValueMakeString(ctx, valueJS), 
						kJSPropertyAttributeDontDelete | kJSPropertyAttributeReadOnly, &exception);

	JSStringRelease(nameJS);
	JSStringRelease(valueJS);
}


//
// Match one and return string or null
//
- (id)match:(NSString*)string withPattern:(NSString*)pattern
{
	return [self match:string withPattern:pattern isCaseSensitive:YES];
}
- (id)match:(NSString*)string withPattern:(NSString*)pattern isCaseSensitive:(BOOL)c
{
	NSString* flags = c ? @"" : @"i";

	[self setConstantStringValue:string forName:@"string"];
	[self setConstantStringValue:pattern forName:@"pattern"];
	[self setConstantStringValue:flags forName:@"flags"];
	
	// Evaluate script
    JSValueRef result = JSEvaluateScript(ctx, matchScriptJS, NULL, NULL, 0, NULL);
	if (!result)	return	nil;
	JSStringRef resultStringJS = JSValueToStringCopy(ctx, result, NULL);
	CFStringRef resultString = JSStringCopyCFString(kCFAllocatorDefault, resultStringJS);
	JSStringRelease(resultStringJS);
	return	(id)resultString;
}

//
// Match all and return array or null
//
- (id)matchAll:(NSString*)string withPattern:(NSString*)pattern
{
	return [self matchAll:string withPattern:pattern isCaseSensitive:YES];
}
- (id)matchAll:(NSString*)string withPattern:(NSString*)pattern isCaseSensitive:(BOOL)c
{
	NSString* flags = c ? @"g" : @"gi";

	[self setConstantStringValue:string forName:@"string"];
	[self setConstantStringValue:pattern forName:@"pattern"];
	[self setConstantStringValue:flags forName:@"flags"];
	
	// Evaluate script
    JSValueRef result = JSEvaluateScript(ctx, matchScriptJS, NULL, NULL, 0, NULL);
	if (!result)	return	nil;

	JSType	type = JSValueGetType(ctx, result);
	// Array
	if (type == kJSTypeObject)
	{
		JSObjectRef arrayJS = JSValueToObject(ctx, result, nil);

		JSStringRef lengthJS = JSStringCreateWithCFString((CFStringRef)@"length");
		JSValueRef lengthValueJS = JSObjectGetProperty(ctx, arrayJS, lengthJS, nil);

		int length = JSValueToNumber(ctx, lengthValueJS, nil);
		
		NSMutableArray* array = [NSMutableArray array];
		int i;
		for (i=0; i<length; i++)
		{
			JSValueRef elt = JSObjectGetPropertyAtIndex(ctx, arrayJS, i, nil);

			JSStringRef stringMatchJS = JSValueToStringCopy(ctx, elt, nil);
			CFStringRef stringMatch = JSStringCopyCFString(kCFAllocatorDefault, stringMatchJS);
			JSStringRelease(stringMatchJS);
			[array addObject:(NSString*)stringMatch];
		}	
		
		JSStringRelease(lengthJS);
		return array;
	}

	return	nil;
}

//
// Replace all and return replaced string
//
- (id)replace:(NSString*)string withString:(NSString*)replacement andPattern:(NSString*)pattern
{
	return	[self replace:string withString:replacement andPattern:pattern isCaseSensitive:YES];
}
- (id)replace:(NSString*)string withString:(NSString*)replacement andPattern:(NSString*)pattern isCaseSensitive:(BOOL)c
{
	NSString* flags = c ? @"g" : @"gi";

	[self setConstantStringValue:string forName:@"string"];
	[self setConstantStringValue:replacement forName:@"replacement"];
	[self setConstantStringValue:pattern forName:@"pattern"];
	[self setConstantStringValue:flags forName:@"flags"];
	
	// Evaluate script
    JSValueRef result = JSEvaluateScript(ctx, replaceScriptJS, NULL, NULL, 0, NULL);
	if (!result)	return	nil;

	JSStringRef resultStringJS = JSValueToStringCopy(ctx, result, NULL);
	CFStringRef resultString = JSStringCopyCFString(kCFAllocatorDefault, resultStringJS);
	JSStringRelease(resultStringJS);
	return	(id)resultString;
}



//
// Match all and return indices matched in string : [[start1, end1], [start2, end2], …] 
//
- (id)indicesFromMatching:(NSString*)string withPattern:(NSString*)pattern
{
	//
	// Setup JS environment : 
	//	string to be matched
	//	pattern
	//	pattern flags
	//
	[self setConstantStringValue:string forName:@"string"];
	[self setConstantStringValue:pattern forName:@"pattern"];
	[self setConstantStringValue:@"gi" forName:@"flags"];
	
	// Evaluate script
    JSValueRef result = JSEvaluateScript(ctx, indicesScriptJS, NULL, NULL, 0, NULL);
	
	// We got an error, bail
	if (!result)	return	nil;

	JSType	type = JSValueGetType(ctx, result);
	// Array
	if (type == kJSTypeObject)
	{
		JSObjectRef arrayJS = JSValueToObject(ctx, result, nil);

		JSStringRef lengthJS = JSStringCreateWithCFString((CFStringRef)@"length");
		JSValueRef lengthValueJS = JSObjectGetProperty(ctx, arrayJS, lengthJS, nil);

		int length = JSValueToNumber(ctx, lengthValueJS, nil);
		
		NSMutableArray* array = [NSMutableArray array];
		int i;
		for (i=0; i<length; i++)
		{
			JSValueRef elt = JSObjectGetPropertyAtIndex(ctx, arrayJS, i, nil);
			JSObjectRef arrayJS2 = JSValueToObject(ctx, elt, nil);
			lengthValueJS = JSObjectGetProperty(ctx, arrayJS2, lengthJS, nil);

			JSValueRef v1 = JSObjectGetPropertyAtIndex(ctx, arrayJS2, 0, nil);
			JSValueRef v2 = JSObjectGetPropertyAtIndex(ctx, arrayJS2, 1, nil);
			int a = JSValueToNumber(ctx, v1, nil);
			int b = JSValueToNumber(ctx, v2, nil);
			
			[array addObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:a], [NSNumber numberWithInt:b], nil]];
		}	
		
		JSStringRelease(lengthJS);
		return array;
	}

	return	nil;
}


@end




//
// NSString RegEx additions
//
@implementation NSString(RegExViaJavascriptCore)

// Match one and return string or null
- (id)matchWithPattern:(NSString*)pattern
{
	return [[RegExJSController sharedController] match:self withPattern:pattern isCaseSensitive:YES];
}
- (id)matchWithPattern:(NSString*)pattern isCaseSensitive:(BOOL)c
{
	return [[RegExJSController sharedController] match:self withPattern:pattern isCaseSensitive:c];
}

// Match all and return array or null
- (id)matchAllWithPattern:(NSString*)pattern
{
	return [[RegExJSController sharedController] matchAll:self withPattern:pattern isCaseSensitive:YES];
}
- (id)matchAllWithPattern:(NSString*)pattern isCaseSensitive:(BOOL)c
{
	return [[RegExJSController sharedController] matchAll:self withPattern:pattern isCaseSensitive:c];
}

// Replace all
- (id)replaceWithString:(NSString*)replacement andPattern:(NSString*)pattern
{
	return [[RegExJSController sharedController] replace:self withString:replacement andPattern:pattern isCaseSensitive:YES];
}
- (id)replaceWithString:(NSString*)replacement andPattern:(NSString*)pattern isCaseSensitive:(BOOL)c
{
	return [[RegExJSController sharedController] replace:self withString:replacement andPattern:pattern isCaseSensitive:c];
}

@end